searchData={"items":[{"type":"module","title":"Tamnoon","doc":"This module provides functions needed to initialize Tamnoon. You do not need to handle\nit directly, rather, the only time you need to call something in this module is in your\nsupervision tree, to add it to the children and configure it (see `child_spec/1`).","ref":"Tamnoon.html"},{"type":"module","title":"Example - Tamnoon","doc":"```\ndef start_link(opts \\\\ []) do\n  children = [Tamnoon]\n  opts = [strategy: :one_for_one, name: Tamnoon.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"Tamnoon.html#module-example"},{"type":"function","title":"Tamnoon.child_spec/1","doc":"Returns a Tamnoon server supervisor child spec. See `t:tamnoon_opts/0` for more info.","ref":"Tamnoon.html#child_spec/1"},{"type":"function","title":"Tamnoon.start_link/1","doc":"Starts the supervisor. See `t:tamnoon_opts/0` for more info.","ref":"Tamnoon.html#start_link/1"},{"type":"type","title":"Tamnoon.tamnoon_opts/0","doc":"Options for initializing Tamnoon. Defaults to `[4000, Tamnoon.Router, Tamnoon.SocketHandler, Tamnoon.Methods, %{}]`.","ref":"Tamnoon.html#t:tamnoon_opts/0"},{"type":"module","title":"Tamnoon.MethodManager","doc":"This module handles the management of different methods as you create them.\r\nNotably, it provides the `defmethod/2` macro.\r\n> #### Importing the module {: .info}\r\n> In order to create handlers for the methods you set up, you must `import Tamnoon.MethodManager`\r\n> in your _methods module_. Then, you can use the `defmethod/2` macro to implement handling\r\n> of the methods.","ref":"Tamnoon.MethodManager.html"},{"type":"macro","title":"Tamnoon.MethodManager.defmethod/2","doc":"Defines a function named `tmnn_[name]`. Functions with this prefix in your _methods module_\r\nwill automatically be added to the possible methods when invoking `route_request/3`.\r\nInside the created function, you can access the state of the client and the request\r\nobject by the `state` and `req` variables respectively.\r\nMethod handlers must return a tuple of `{return_value, new_state}`.","ref":"Tamnoon.MethodManager.html#defmethod/2"},{"type":"macro","title":"Example - Tamnoon.MethodManager.defmethod/2","doc":"```\r\ndefmethod :get do\r\n  key = get_key(req, state)\r\n  if (key != nil) do\r\n    {state[key], state}\r\n  else\r\n    {\"Error: no matching key\", state}\r\n  end\r\nend\r\n```","ref":"Tamnoon.MethodManager.html#defmethod/2-example"},{"type":"function","title":"Tamnoon.MethodManager.route_request/3","doc":"The function used by `Tamnoon.SocketHandler.websocket_handle/2` to route the requests\r\nto the appropriate method handler.","ref":"Tamnoon.MethodManager.html#route_request/3"},{"type":"module","title":"Tamnoon.Methods","doc":"Provides default implementations for methods you need to get your Tamnoon server\r\nworking - basic state management (with `tmnn_get/2` and `tmnn_update/2`) and basic handling of\r\nPubSub channels (via `tmnn_sub/2`, `tmnn_unsub/2`, `tmnn_pub/2`, `tmnn_subbed_channels/2`)\r\n> #### Using the module {: .info}\r\n> When you `use Tamnoon.Methods`, every method in this module will be imported for you,\r\n> letting you conveniently make a _methods module_. See `m:Tamnoon.MethodManager` for\r\n> details on making one.\r\n\r\n_Note: in the documentation of the default methods, the term 'returns' is used both\r\nto describe the return value of the function and the value of the server's response._","ref":"Tamnoon.Methods.html"},{"type":"function","title":"Tamnoon.Methods.get_key/2","doc":"Returns the value of the key under the field `\"key\"` in the request as an atom.\r\nNote: this is *NOT* a method handler - simply a utility function.","ref":"Tamnoon.Methods.html#get_key/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_get/2","doc":"Returns the field with the key specified under the `\"key\"` field\r\nin the request. Returns an error string if there is no such item.","ref":"Tamnoon.Methods.html#tmnn_get/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_pub/2","doc":"Will send the request under the `\"action\"` key in the request to every client\r\nin the channel under the `\"channel\"` key.","ref":"Tamnoon.Methods.html#tmnn_pub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_sub/2","doc":"Subscribes to the channel under the `\"key\"` field in the request. If there is no such\r\nchannel, one will be created (with the client subscribed to it).\r\nIf the client is already subscribed to the channel nothing will happen.","ref":"Tamnoon.Methods.html#tmnn_sub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_subbed_channels/2","doc":"Returns a list of the channels the client is currently subscribed to.","ref":"Tamnoon.Methods.html#tmnn_subbed_channels/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_unsub/2","doc":"Unsubscribes from the channel under the `\"key\"` in the request.","ref":"Tamnoon.Methods.html#tmnn_unsub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_update/2","doc":"Updates the value of the item with the key at the `\"key\"` field on the request,\r\nsetting it to the value at the `\"val\"` key. Returns a tuple with the updated field and\r\nthe state, or an error string if there is no such item.","ref":"Tamnoon.Methods.html#tmnn_update/2"},{"type":"module","title":"Tamnoon.Router","doc":"This module provides a default router for HTTP(S) requests, returning a 404 status\r\nto all of them. This is useful in order to only serve Websocket requests, however\r\ncreating a custom one configured to serve a website might be beneficial. If you wish to\r\ndo so, see `m:Plug.Router` and `Tamnoon.child_spec/1`.","ref":"Tamnoon.Router.html"},{"type":"function","title":"Tamnoon.Router.call/2","doc":"","ref":"Tamnoon.Router.html#call/2"},{"type":"function","title":"Tamnoon.Router.init/1","doc":"","ref":"Tamnoon.Router.html#init/1"},{"type":"module","title":"Tamnoon.SocketHandler","doc":"A default `:cowboy_websocket` implementation. Usually there should not be a reason to do anything\r\ndirectly with this module, as it gets its options from `Tamnoon.start_link/1` and\r\nadapts to them automatically. However, it is still documented for the sake of allowing\r\nextensiblity.\r\nYou can replace the module with a custom one via setting it as the _(drumroll..)_ `:socket_handler`\r\nkey's value in `Tamnoon.start_link/1`'s options.","ref":"Tamnoon.SocketHandler.html"},{"type":"function","title":"Tamnoon.SocketHandler.init/2","doc":"Initiates the websocket connection","ref":"Tamnoon.SocketHandler.html#init/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_handle/2","doc":"Receives a websocket request and decodes it, sending it to the `Tamnoon.MethodManager.route_request/3`\r\nfunction.","ref":"Tamnoon.SocketHandler.html#websocket_handle/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_info/2","doc":"Handles messages from other BEAM processes. By default it is only used for handling\r\n`\"pub\"` requests, and as such it simply forwards the message to `websocket_handle/2`.","ref":"Tamnoon.SocketHandler.html#websocket_info/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_init/1","doc":"Initiates the websocket and adds the client to the registry under the `\"clients\"` channel.","ref":"Tamnoon.SocketHandler.html#websocket_init/1"},{"type":"extras","title":"Tamnoon","doc":"# Tamnoon\n\n#### Make Elixir power your favorite front-end framework effortlessly!\n\nThis package/framework is designed to serve as a simple replacement for those big scary frameworks that have a huge learning curve and are waaaaaay (like, waaaaaay) too overkill for simple web apps.\n\nBy utilizing websockets, Tamnoon can provide realtime client-server communication and PubSub functionality (in fact, it comes with this ability out of the box!) without a need to worry about anything.\n\nAnd the best part? It doesn't even matter which front-end technology you want to use. Hell, as long as it can send json over websockets, it doesn't even have to be a web app! Could be a turtle for all I care. _(if you somehow find a turtle that can communicate via websockets, that is..)_","ref":"readme.html"},{"type":"extras","title":"Installation - Tamnoon","doc":"The package can be installed by adding `tamnoon` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:tamnoon, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\r\n\r\nGetting started with Tamnoon takes only a few simple steps. First, create a new project with a supervision tree:\r\n\r\n```console\r\n$ mix new [NAME] --sup\r\n```\r\n\r\nThen, add Tamnoon to your `deps` in `mix.exs`:\r\n\r\n```\r\n  defp deps do\r\n    [\r\n      # Other dependencies...\r\n      {:tamnoon, \"~> 0.1.0\"}\r\n    ]\r\n  end\r\n```\r\n\r\nWe can now get started! Simply add Tamnoon to your supervision tree as such:\r\n\r\n```\r\n  def start(_type, _args) do\r\n    children = [\r\n      {Tamnoon}\r\n    ]\r\n    opts = [strategy: :one_for_one, name: TamnoonTest.Supervisor]\r\n    Supervisor.start_link(children, opts)\r\n  end\r\n```\r\n\r\nYou can now run your project by using `iex -S mix` and bask in the glory of your work:\r\n\r\n```console\r\n11:01:54.386 [info] Tamnoon listening on port 4000..\r\n```\r\n\r\n...Or you can keep reading the documentation to actually be able to make something useful. Up to you :)","ref":"getting-started.html"},{"type":"extras","title":"Basic Usage","doc":"# Basic Usage\r\n\r\nWith Tamnoon aiming to be as simple as possible, you don't need much more than the basic functionality in order to write simple apps. We will be creating a simple app where users can fight a number that is displayed. You can find the finished app on [github](https://github.com/omer-sm/tamnoon_sample).\r\n\r\nFirst, you can set up your _initial state_ by passing a map of the default configuration to the, aptly named, `:initial_state` property of Tamnoon in your `start/2` function:\r\n\r\n```\r\nchildren = [\r\n  {Tamnoon, [[initial_state: %{val: 0}]]}\r\n  #...\r\n]\r\n```\r\n\r\nThis will configure Tamnoon to give a default state with a field `:val` of value 0 to every new client. Note that Tamnoon does not support dynamic states - meaning, our state will only ever have one field - `:val`. \r\n\r\nBelieve it or not, that is basically it for the backend. However we still have no front-end! Usually you can serve your front-end separately, but for this tutorial we will serve it from our app by creating a custom router.\r\n\r\nCreate a new file named `router.ex` in your _lib_ directory, and set it as such:\r\n```\r\ndefmodule TamnoonSample.Router do\r\n  use Plug.Router\r\n\r\n  plug :match\r\n  plug :dispatch\r\n\r\n  get \"/\" do\r\n    send_file(conn, 200, \"priv/static/index.html\")\r\n  end\r\n\r\n  match _ do\r\n    send_resp(conn, 404, \"404\")\r\n  end\r\nend\r\n```\r\n\r\nThe router will now serve the file `index.html` file at `priv/static` - so we need to create the directories and the file. Additionally, we need to tell Tamnoon to use our router instead of the default one. In your `start/2` function, update Tamnoon as such:\r\n\r\n```\r\nchildren = [\r\n  {Tamnoon, [[initial_state: %{val: 0}, router: TamnoonSample.Router]]}\r\n]\r\n```\r\n\r\nNow let's create the html file. We won't go over it here, however you can look at the source code on github. What _is_ important however is the JSON of the requests we will be sending. We want to have our app show the value of `:val` in our state. This can be achieved by sending the following request to our server:\r\n\r\n```\r\n{\r\n    \"method\": \"get\",\r\n    \"key\": \"val\"\r\n}\r\n```\r\n\r\nThis (and the `\"update\"` method) returns a map of `%{key: value}` to the client. As such, we need to set an appropriate handler for this in our html file.\r\n\r\nNow, we can move on to defining methods for incrementing or decrementing our number.","ref":"basic-usage.html"},{"type":"extras","title":"Custom Methods","doc":"# Custom Methods\r\n\r\nAssuming that you've been following the _basic usage_ tutorial, you should have created a basic Tamnoon app that stores a value. Now, what about changing the value?\r\n\r\nWe could use the `\"update\"` method to do that and set it to something specific. For example, sending the following request to the server will set the value to 1:\r\n\r\n```\r\n{\r\n    \"method\": \"update\",\r\n    \"key\": \"val\",\r\n    \"val\": 1\r\n}\r\n```\r\n\r\nHowever, that is really fucking boring. We're here for fun. How about we use the `Tamnoon.MethodManager.defmethod/2` macro to create a _custom method_ instead?\r\n\r\nWe'll start off by making a file called `methods.ex` and having it `use Tamnoon.Methods`. This will set our module up to support every built-in method and let us define additional ones. (see `m:Tamnoon.Methods`)\r\n\r\nWe will now utilize the `Tamnoon.MethodManager.defmethod/2` macro to create a _custom method_ called `\"nudge\"`. This method will take a key and a direction (up/down) and increment / decrement the value of the field. For example, this will increment `\"val\"` by 1.\r\n\r\n```\r\n{\r\n    \"method\": \"nudge\",\r\n    \"key\": \"val\",\r\n    \"direction\": \"up\"\r\n}\r\n```\r\n\r\nNow, let's define our method handler in our `methods.ex` file as following:\r\n\r\n```\r\n  defmethod :nudge do\r\n    key = Tamnoon.Methods.get_key(req, state)\r\n    val = Map.get(state, key)\r\n    new_val = if (req[\"direction\"] == \"up\"), do: val + 1, else: val - 1\r\n    new_state = Map.put(state, key, new_val)\r\n    {%{key => new_val}, new_state}\r\n  end\r\n```\r\n\r\nThis will simply change the value of the request field by 1. We can now test it, and see that it works. However, we want to have our users \"fight\" over the number - and currently that is not happening. If you open two clients at the same time, you will see that each one has their own value, independent of the other. \r\n\r\nFor the value to be shared, we will need to utilize _PubSub_ functionality, which we will go over in the next tutorial.","ref":"custom-methods.html"},{"type":"extras","title":"PubSub","doc":"# PubSub\r\n\r\nThis guide assumes that you have been following the previous guides. You can follow along if you haven't too, however you may want to either skim over the previous ones or take a look at the [sample app's source](https://github.com/omer-sm/tamnoon_sample).","ref":"pubsub.html"},{"type":"extras","title":"Channels - PubSub","doc":"Tamnoon's PubSub functionality relies on _channels_. By default, every new client is added to the `\"clients\"` channel. Channels can be subscribed to, unsubscribed to, and published to.","ref":"pubsub.html#channels"},{"type":"extras","title":"Subscribing and Unsubscribing - PubSub","doc":"By utilizing the `\"sub\"` and `\"unsub\"` methods (see `m:Tamnoon.Methods`), clients can subscribe and unsubscribe to channels. A client that is subscribed to a channel will execute any action that is _published_ to that channel.\r\n\r\n_Note: unlike the state, channels are dynamically created. This means that when subscribing to a channel that does not exist, it will be created._","ref":"pubsub.html#subscribing-and-unsubscribing"},{"type":"extras","title":"Publishing - PubSub","doc":"By _publishing_ to a channel, a client will cause any subscriber of that channel to execute the action requested. For example:\r\n\r\n```\r\n{\r\n    \"method\": \"pub\",\r\n    \"channel\": \"clients\",\r\n    \"action\": {\r\n        \"method\": \"update\",\r\n        \"key\": \"val\",\r\n        \"val\": 1\r\n    }\r\n}\r\n```\r\n\r\nThe above request will cause every client (as it is published to the `\"clients\"` channel) to update the value of `\"val\"` to 1. This will trigger a message from the server to the client specifying the new value, as with simply using the `\"update\"` method.","ref":"pubsub.html#publishing"},{"type":"extras","title":"Synchronizing Our Clients - PubSub","doc":"Now, let's say we utilized our `\"nudge\"` method from the previous guide and made buttons to increment / decrement the number. Currently, the button to increment will send the following JSON:\r\n\r\n```\r\n{\r\n    \"method\": \"nudge\",\r\n    \"key\": \"val\",\r\n    \"direction\": \"up\"\r\n}\r\n```\r\n\r\nTo make it change the number of _every_ client, we can simply wrap it in a `\"pub\"`:\r\n\r\n```\r\n{\r\n    \"method\": \"pub\",\r\n    \"channel\": \"clients\",\r\n    \"action\": {\r\n        \"method\": \"nudge\",\r\n        \"key\": \"val\",\r\n        \"direction\": \"up\"\r\n    }\r\n}\r\n```\r\n\r\nNow we can open two clients and see it in action. We can nudge the value up and down, and it updates automatically. However, if we change the number before opening another client, the numbers themselves won't be the same - they will just change at the same time. This can be solved, for example, by using an agent to hold a global value, or by \r\nmaking a custom method that will cause other clients to publish an `\"update\"` action with their value. However, this is enough for now.\r\n\r\nSoooo... that's it! Pretty simple right? You are now ready to start making real-time web apps with Elixir and your favorite front-end technology. Go wild!","ref":"pubsub.html#synchronizing-our-clients"},{"type":"extras","title":"Troubleshooting / Common Errors","doc":"# Troubleshooting / Common Errors\r\n\r\nThis section contains help with common errors that may arise while using Tamnoon.","ref":"troubleshooting.html"},{"type":"extras","title":"Jason.Encoder must be explicitly implemented - Troubleshooting / Common Errors","doc":"This error may arise while using the `Tamnoon.MethodManager.defmethod/2` macro. Notably, this will happen when the value that the method handler returns to the client is not able to be decoded by `m:Jason`. \r\nTo fix this, always return maps to the client. For example, this will NOT work:\r\n\r\n```\r\ndefmethod :my_method do\r\n  # Do something...\r\n  {{key, value}, state}\r\nend\r\n```\r\n\r\nThis WILL work:\r\n\r\n```\r\ndefmethod :my_method do\r\n  # Do something...\r\n  {%{key => value}, state}\r\nend\r\n```","ref":"troubleshooting.html#jason-encoder-must-be-explicitly-implemented"},{"type":"extras","title":"Method Overview","doc":"# Method Overview\r\n\r\nThis guide is a simple collection of examples for the JSON requests the default methods accept and the JS objects they return. (see `m:Tamnoon.Methods`)","ref":"method-overview.html"},{"type":"extras","title":"get - Method Overview","doc":"```\r\n{\r\n    \"method\": \"get\",\r\n    \"key\": \"your key\"\r\n}\r\n```\r\n_Returns: {key: value} or {error: \"Error: no matching key\"}_","ref":"method-overview.html#get"},{"type":"extras","title":"update - Method Overview","doc":"```\r\n{\r\n    \"method\": \"update\",\r\n    \"key\": \"your key\",\r\n    \"val\": \"your new value\"\r\n}\r\n```\r\n_Returns: {key: value} or {error: \"Error: no matching key\"}_","ref":"method-overview.html#update"},{"type":"extras","title":"sub - Method Overview","doc":"```\r\n{\r\n    \"method\": \"sub\",\r\n    \"channel\": \"your channel\"\r\n}\r\n```\r\n_Returns: {sub: :ok}_","ref":"method-overview.html#sub"},{"type":"extras","title":"unsub - Method Overview","doc":"```\r\n{\r\n    \"method\": \"unsub\",\r\n    \"channel\": \"your channel\"\r\n}\r\n```\r\n_Returns: {unsub: :ok} or {error: \"Error: can't unsub from clients channel\"}_","ref":"method-overview.html#unsub"},{"type":"extras","title":"pub - Method Overview","doc":"```\r\n{\r\n    \"method\": \"pub\",\r\n    \"channel\": \"your channel\",\r\n    \"action\": {\r\n        \"method\": \"any method\",\r\n        ...\r\n    }\r\n}\r\n```\r\n_Returns: {pub: :ok}_","ref":"method-overview.html#pub"},{"type":"extras","title":"subbed_channels - Method Overview","doc":"```\r\n{\r\n    \"method\": \"subbed_channels\"\r\n}\r\n```\r\n_Returns: {subbed\\_channels: channels\\_array}_","ref":"method-overview.html#subbed_channels"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}